@@ -1,10 +1,13 @@
-lognet=function(x,is.sparse,ix,jx,y,weights,offset,alpha,nobs,nvars,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,intr,vnames,maxit,kopt,family){
+lognet=function(x,is.sparse,y,weights,offset,alpha,nobs,nvars,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,intr,vnames,maxit,kopt,family,pb){
   nc=dim(y)
   maxit=as.integer(maxit)
   if(is.null(nc)){
     ## Need to construct a y matrix, and include the weights
     y=as.factor(y)
     ntab=table(y)
+    minclass=min(ntab)
+    if(minclass<=1)stop("one multinomial or binomial class has 1 or 0 observations; not allowed")
+    if(minclass<8)warning("one multinomial or binomial class has fewer than 8  observations; dangerous ground")
     classnames=names(ntab)
     nc=as.integer(length(ntab))
     y=diag(nc)[as.numeric(y),]
@@ -15,7 +18,9 @@ lognet=function(x,is.sparse,ix,jx,y,weights,offset,alpha,nobs,nvars,jd,vp,cl,ne,
     nc=as.integer(nc[2])
     classnames=colnames(y)
   }
-
+#Check for size limitations
+  maxvars=.Machine$integer.max/(nlam*nc)
+  if(nx>maxvars)stop(paste("Integer overflow; num_classes*num_lambda*pmax should not exceed .Machine$integer.max. Reduce pmax to be below", trunc(maxvars)))
 
   if(!missing(weights)) y=y*weights
 ### check if any rows of y sum to zero, and if so deal with them
@@ -26,22 +31,23 @@ lognet=function(x,is.sparse,ix,jx,y,weights,offset,alpha,nobs,nvars,jd,vp,cl,ne,
     x=x[o,,drop=FALSE]
     nobs=sum(o)
   }else o=NULL
-    
+
 
   if(family=="binomial"){
     if(nc>2)stop("More than two classes; use multinomial family instead in call to glmnet",call.=FALSE)
     nc=as.integer(1) # for calling lognet
-    y=y[,c(2,1)]#fortran lognet models the first column; we prefer the second (for 0/1 data)
+    y=y[,c(2,1)]#C++ lognet models the first column; we prefer the second (for 0/1 data)
   }
 
-  
+
     storage.mode(y)="double"
     if(is.null(offset)){
       offset=y*0 #keeps the shape of y
-      is.offset=FALSE}
+      is.offset=FALSE
+    }
     else{
       offset=as.matrix(offset)
-      if(!is.null(o))offset=offset[o,]# we might have zero weights
+      if(!is.null(o))offset=offset[o,,drop=FALSE]# we might have zero weights
       do=dim(offset)
       if(do[[1]]!=nobs)stop("offset should have the same number of values as observations in binomial/multinomial call to glmnet",call.=FALSE)
       if((do[[2]]==1)&(nc==1))offset=cbind(offset,-offset)
@@ -49,10 +55,15 @@ lognet=function(x,is.sparse,ix,jx,y,weights,offset,alpha,nobs,nvars,jd,vp,cl,ne,
       storage.mode(offset)="double"
       is.offset=TRUE
     }
-    fit=if(is.sparse) .Fortran("splognet",
-      parm=alpha,nobs,nvars,nc,x,ix,jx,y,offset,jd,vp,cl,ne=ne,nx,nlam,flmin,ulam,thresh,isd,intr,maxit,kopt,
+
+    # C++ expects offset to have nc number of columns, as documented in Fortran.
+    # We do a final modification of offset to take only the first column, since we favor this when fitting.
+    if (nc == 1) offset=as.matrix(offset[,1], ncol=1)
+
+    fit=if(is.sparse) splognet_exp(
+      parm=alpha,x,y,offset,jd,vp,cl,ne=ne,nx,nlam,flmin,ulam,thresh,isd,intr,maxit,kopt,pb,
                     lmu=integer(1),
-                    a0=double(nlam*nc),
+                    a0=matrix(0.0, nc, nlam),
                     ca=double(nx*nlam*nc),
                     ia=integer(nx),
                     nin=integer(nlam),
@@ -60,20 +71,20 @@ lognet=function(x,is.sparse,ix,jx,y,weights,offset,alpha,nobs,nvars,jd,vp,cl,ne,
                     dev=double(nlam),
                     alm=double(nlam),
                     nlp=integer(1),
-                    jerr=integer(1),PACKAGE="glmnet"
+                    jerr=integer(1)
                     )
-  else .Fortran("lognet",
-          parm=alpha,nobs,nvars,nc,as.double(x),y,offset,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,intr,maxit,kopt,
+  else lognet_exp(
+          parm=alpha,x,y,offset,jd,vp,cl,ne,nx,nlam,flmin,ulam,thresh,isd,intr,maxit,kopt,pb,
           lmu=integer(1),
-          a0=double(nlam*nc),
+          a0=matrix(0.0, nc, nlam),
           ca=double(nx*nlam*nc),
           ia=integer(nx),
           nin=integer(nlam),
-          nulldev=double(1),      
+          nulldev=double(1),
           dev=double(nlam),
           alm=double(nlam),
           nlp=integer(1),
-          jerr=integer(1),PACKAGE="glmnet"
+          jerr=integer(1)
           )
 if(fit$jerr!=0){
   errmsg=jerr(fit$jerr,maxit,pmax=nx,family)
