@@ -40,7 +40,7 @@ predict.multnet <-
     }
   }
   else {
-    for (i in seq(nclass)) nbeta[[i]] = methods::rbind2(a0[i, ], nbeta[[i]])#was rbind2
+    for (i in seq(nclass)) nbeta[[i]] = methods::rbind2(a0[i,,drop=FALSE ], nbeta[[i]])#was rbind2
     nlambda=length(object$lambda)
   }
   if (type == "coefficients")
@@ -53,7 +53,7 @@ predict.multnet <-
                                                    drop = FALSE], bystep = TRUE)))
   dd = dim(newx)
    if (inherits(newx, "sparseMatrix"))
-    newx = as(newx, "dgCMatrix")
+    newx = as(newx, "dMatrix")
   npred = dd[[1]]
   dn = list(names(nbeta), dimnames(nbeta[[1]])[[2]], dimnames(newx)[[1]])
    dp = array(0, c(nclass, nlambda, npred), dimnames = dn)
